{# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved #}
{{ header }}
from dataclasses import dataclass, field
{% for imp in imports %}
{{ imp }}
{% endfor %}
{% for class_name in classes %}


@dataclass
class {{class_name}}Conf:
    _target_: str = "{{ classes_map[class_name].target }}"
{% for param in classes_map[class_name].parameters %}
    {{ param.name }}: {{ param.type_str }} = {{ param.default }}
{% endfor %}
{% endfor %}
